############################################################
# wxPyCalc-refactored-v2.py
#
# Created 11/11/2008 by Mike Driscoll
#
# Description: A basic calculator
############################################################

import wx

class MainPanel(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent=parent, id=wx.ID_ANY)
        self.parent = parent
        self.formula = []
        self.currentVal = "0"
        self.previousVal = "0"
        self.operator = None
        self.operatorFlag = False
        self.createAndlayoutWidgets()
        
    def createAndlayoutWidgets(self):
        mainSizer = wx.BoxSizer(wx.VERTICAL)
        masterBtnSizer = wx.BoxSizer(wx.HORIZONTAL)
        vBtnSizer = wx.BoxSizer(wx.VERTICAL)
        numSizer  = wx.GridBagSizer(hgap=5, vgap=5)
        
        self.displayTxt = wx.TextCtrl(self, wx.ID_ANY, "0", 
                                      size=(155,-1),
                                      style=wx.TE_RIGHT|wx.TE_READONLY)
        # number buttons
        size=(45, 45)
        zeroBtn = self.onWidgetSetup(wx.Button(self, wx.ID_ANY, "0", size=size),
                                     wx.EVT_BUTTON, self.onButton, numSizer,
                                     pos=(4,1))
        oneBtn = self.onWidgetSetup(wx.Button(self, wx.ID_ANY, "1", size=size),
                                    wx.EVT_BUTTON, self.onButton, numSizer,
                                    pos=(3,0))
        twoBtn = self.onWidgetSetup(wx.Button(self, wx.ID_ANY, "2", size=size),
                                    wx.EVT_BUTTON, self.onButton, numSizer,
                                    pos=(3,1))
        threeBtn = self.onWidgetSetup(wx.Button(self, wx.ID_ANY, "3", size=size),
                                      wx.EVT_BUTTON, self.onButton, numSizer,
                                      pos=(3,2))
        fourBtn = self.onWidgetSetup(wx.Button(self, wx.ID_ANY, "4", size=size),
                                     wx.EVT_BUTTON, self.onButton, numSizer,
                                     pos=(2,0))
        fiveBtn = self.onWidgetSetup(wx.Button(self, wx.ID_ANY, "5", size=size),
                                     wx.EVT_BUTTON, self.onButton, numSizer,
                                     pos=(2,1))
        sixBtn = self.onWidgetSetup(wx.Button(self, wx.ID_ANY, "6", size=size),
                                    wx.EVT_BUTTON, self.onButton, numSizer,
                                    pos=(2,2))
        sevenBtn = self.onWidgetSetup(wx.Button(self, wx.ID_ANY, "7", size=size),
                                      wx.EVT_BUTTON, self.onButton, numSizer,
                                      pos=(1,0))
        eightBtn = self.onWidgetSetup(wx.Button(self, wx.ID_ANY, "8", size=size),
                                      wx.EVT_BUTTON, self.onButton, numSizer,
                                      pos=(1,1))
        nineBtn = self.onWidgetSetup(wx.Button(self, wx.ID_ANY, "9", size=size),
                                     wx.EVT_BUTTON, self.onButton, numSizer,
                                     pos=(1,2))
        # operator buttons
        divBtn = self.onWidgetSetup(wx.Button(self, wx.ID_ANY, "/", size=size),
                                    wx.EVT_BUTTON, self.onOperation, numSizer,
                                    pos=(0,0), flags=wx.CENTER)
        multiBtn = self.onWidgetSetup(wx.Button(self, wx.ID_ANY, "*", size=size),
                                      wx.EVT_BUTTON, self.onOperation, numSizer,
                                      pos=(0,1), flags=wx.CENTER)
        subBtn = self.onWidgetSetup(wx.Button(self, wx.ID_ANY, "-", size=size),
                                    wx.EVT_BUTTON, self.onOperation, numSizer,
                                    pos=(0,2), flags=wx.CENTER)
        addBtn = self.onWidgetSetup(wx.Button(self, wx.ID_ANY, "+", size=(45,100)),
                                    wx.EVT_BUTTON, self.onOperation, vBtnSizer)
        equalsBtn = self.onWidgetSetup(wx.Button(self, wx.ID_ANY, "Enter", size=(45,100)),
                                       wx.EVT_BUTTON, self.onCalculate, vBtnSizer)
        masterBtnSizer.Add(numSizer, 0, wx.ALL, 5)
        masterBtnSizer.Add(vBtnSizer, 0, wx.ALL, 5)
        mainSizer.Add(self.displayTxt, 0, wx.ALL, 5)
        mainSizer.Add(masterBtnSizer)
        self.SetSizer(mainSizer)
        mainSizer.Fit(self.parent)
        
    def onWidgetSetup(self, widget, event, handler, sizer, pos=None, flags=[]):
        """
        Accepts a widget, the widget's default event and its handler,
        the sizer for the widget, the position of the widget inside 
        the sizer (if applicable) and the sizer flags (if applicable)
        """
        widget.Bind(event, handler)        
        if not pos:
            sizer.Add(widget, 0, wx.ALL, 5)
        elif pos and flags:
            sizer.Add(widget, pos=pos, flag=wx.CENTER) 
        else:
            sizer.Add(widget, pos=pos)
        
        return widget
        
    def onOperation(self, event):
        """
        Add an operator to the equation
        """
        print "onOperation handler fired"

    def onButton(self, event):
        """
        Keeps the display up to date
        """
        # Get the button object
        buttonObj = event.GetEventObject()
        # Get the label of the button object
        buttonLbl = buttonObj.GetLabel()     
        
        self.onNumber(buttonLbl)
        
    def onCalculate(self):
        """
        Calculate the total
        """
        print 'in onCalculate'

class PyCalcFrame(wx.Frame):
    def __init__(self):
        wx.Frame.__init__(self, parent=None, id=wx.ID_ANY, 
                          title="Calculator")
        panel = MainPanel(self)
                        
class PyCalc(wx.App):
    def __init__(self, redirect=False, filename=None):
        wx.App.__init__(self, redirect, filename)
        
    def OnInit(self):
        # create frame here
        frame = PyCalcFrame()
        frame.Show()
        return True
    
def main():
    app = PyCalc()
    app.MainLoop()
    
if __name__ == "__main__":
    main()
